try:
    from importlib.metadata import version
except ImportError:
    from importlib_metadata import version

try:
    __version__ = version("ogameasure")
except:
    __version__ = "0.0.0"


from .communicator import *
from .device import *



from .. import device

class tpg261(device.device):
    manufacturer = 'Pfeiffer Vacuum'
    product_name = 'TPG261'
    classification = 'Vacuum Gauge'

    def _command(self, cmd):
        self.com.send(cmd)
        self._check_response()
        return

    def _query(self, cmd):
        self.com.send(cmd)
        self._check_response()
        self.com.send('\x05')
        d = self.com.readline().strip()
        if d.find(',') != -1: return d.split(',')
        return d
    
    def _check_response(self):
        res = self.com.readline().strip()
        if res == '\x06':
            return
        elif res == '\x15':
            self.com.send('\x05')
            error_no = self.com.readline().strip()
            error_msgs = []
            if error_no[0] == '1': error_msgs.append('Controller error')
            if error_no[1] == '1': error_msgs.append('No hardware')
            if error_no[2] == '1': error_msgs.append('Inadmissible parameter')
            if error_no[3] == '1': error_msgs.append('Syntax error')
            raise Exception(f'Error: error_code = {error_no}, msg = {error_msgs}.')
        
        raise Exception(f'Error: unexpected response. res = {res}.')
    
    def read_single_pressure(self, ch=1):
        d = self._query(f'PR{ch}')
        return self._decode_pr_reply(d[0], d[1])

    def read_pressure(self):
        d = self._query('PRX')
        return {
            'ch1': self._decode_pr_reply(d[0], d[1]),
            'ch2': self._decode_pr_reply(d[2], d[3]),
        }

    def _decode_pr_reply(self, d0, d1):
        status_msg = {
            '0': 'OK',
            '1': 'Underrange',
            '2': 'Overrange',
            '3': 'Sensor error',
            '4': 'Sensor off',
            '5': 'No sensor',
            '6': 'Identification error',
        }
        
        return {
            'status_code': int(d0),
            'status': status_msg[d0],
            'value': float(d1),
        }

    def start_continuous_output(self, mode=1):
        self._command(f'COM,{mode}')
        return
    
    def wait_and_receive_pressure_output(self):
        d = self.com.readline().strip().split(',')
        return {
            'ch1': self._decode_pr_reply(d[0], d[1]),
            'ch2': self._decode_pr_reply(d[2], d[3]),
        }

    def stop_continous_output(self):
        self.com.send('\x05')
        self.com.recv(1000000)
        return
    
    def gauge_status_query(self):
        return self.gauge_status_command()

    def gauge_status_command(self, ch1=0, ch2=0):
        d = self._query(f'SEN,{ch1},{ch2}')
        
        status_msg = {
            '0': 'unavailable',
            '1': 'off',
            '2': 'on',
        }
        return {
            'ch1': {
                'code': int(d[0]),
                'message': status_msg[d[0]],
            },
            'ch2': {
                'code': int(d[1]),
                'message': status_msg[d[1]],
            },
        }
    
    def gauge_identification_query(self):
        d = self._query('TID')

        id_msg = {
            'TPR': 'Pirani Gauge',
            'IKR9': 'Cold Cathode Gauge 1e-9',
            'IKR11': 'Cold Cathode Gauge 1e-11',
            'PKR': 'FullRange CC Gauge',
            'PBR': 'FullRange BA Gauge',
            'IMR': 'Pirani / High Pressure Gauge',
            'CMR': 'Linear gauge',
            'noSen': 'no Sensor',
            'noid': 'no identifier',
        }
        return {
            'ch1': {
                'id': d[0],
                'description': id_msg[d[0]],
            },
            'ch2': {
                'id': d[1],
                'description': id_msg[d[1]],
            }
        }

    def display_channel_command(self, ch=None):
        if ch is None:
            d = self._query('SCT')
        else:
            d = self._query(f'SCT,{ch}')
            pass
        
        display_msg = {
            '0': 'Gauge 1',
            '1': 'Gauge 2',
        }
        return {
            'code': int(d),
            'description': display_msg[d],
        }

    def display_channel_query(self):
        return self.display_channel_command()

    def reset_error_command(self):
        d = self._query('RES,1')
        return self._decode_res(d)
        
    def error_query(self):
        d = self._query('RES')
        return self._decode_res(d)

    def _decode_res(self, errors):
        msg = {
            '0': 'No error',
            '1': 'Watchdog has responded',
            '2': 'Task fail error',
            '3': 'EPROM error',
            '4': 'RAM error',
            '5': 'EEPROM error',
            '6': 'Display error',
            '7': 'A/D converter error',
            '8': '',
            '9': 'Gauge 1 error (e.g., filament rupture, no supply)',
            '10': 'Gauge 1 identification error',
            '11': 'Gauge 2 error (e.g., filament rupture, no supply)',
            '12': 'Gauge 2 identification error',
        }

        ret = []
        for e in errors.split(','):
            ret.append({
                'code': int(e),
                'message': msg[e],
            })
            continue
        return ret

    def unit_query(self):
        return self.unit_command()

    def unit_command(self, unit=None):
        if unit is None:
            d = self._query('UNI')
        else:
            d = self._query(f'UNI,{unit}')
            pass
        
        desc = {
            '0': 'mbar/bar',
            '1': 'Torr',
            '2': 'Pascal',
        }
        return {
            'code': int(d),
            'unit': desc[d],
        }

    def display_resolution_query(self):
        return self.display_resolution_command()
    
    def display_resolution_command(self, res=None):
        if res is None:
            d = self._query('DCD')
        else:
            d = self._query(f'DCD,{res}')
            pass

        desc = {
            '2': 'Display x.x (2 digits)',
            '3': 'Display x.xx (3 digits)',
        }
        return {
            'code': int(d),
            'description': desc[d],
        }

    def firmware_version_query(self):
        return self._query('PNR')
    



#! /usr/bin/env python3
import serial
import time,sys,os
import math
import datetime

class ND287(object):

    def __init__(self, port):
        self._enc = self.serial_open(port)
        return

    def serial_open(self, port):
        _enc = serial.Serial(port,parity="E",stopbits=2, bytesize=7)
        return _enc

    def write(self, cmd):
        self._enc.write(cmd)
        return

    def read(self, byte=100):
        res = self._enc.readline()
        return res

    def query(self,cmd,wt=0.001):
        self.write(cmd)
        time.sleep(wt)
        ret = self.read()
        return ret

    def press_key(self,key):
        if key == "0":
            self.write(b"\x1BT0000\r")
        elif key == "1":
            self.write(b"\x1BT0001\r")
        elif key == "2":
            self.write(b"\x1BT0002\r")
        elif key == "3":
            self.write(b"\x1BT0003\r")
        elif key == "4":
            self.write(b"\x1BT0004\r")
        elif key == "5":
            self.write(b"\x1BT0005\r")
        elif key == "6":
            self.write(b"\x1BT0006\r")
        elif key == "7":
            self.write(b"\x1BT0007\r")
        elif key == "8":
            self.write(b"\x1BT0008\r")
        elif key == "9":
            self.write(b"\x1BT0009\r")
        elif key == "CLR":
            self.write(b"\x1BT0100\r")
        elif key == "-":
            self.write(b"\x1BT0101\r")
        elif key == ".":
            self.write(b"\x1BT0102\r")
        elif key == "NAVI":
            self.write(b"\x1BT0103\r")
        elif key == "ENT":
            self.write(b"\x1BT0004\r")
        elif key == "up":
            self.write(b"\x1BT0005\r")
        elif key == "down":
            self.write(b"\x1BT0006\r")
        elif key == "soft1":
            self.write(b"\x1BT0007\r")
        elif key == "soft2":
            self.write(b"\x1BT0008\r")
        elif key == "soft3":
            self.write(b"\x1BT0009\r")
        elif key == "soft4":
            self.write(b"\x1BT0010\r")
        else:
            pass

    def output_device_identification(self):
        """
        Return
        1 Device nameID
        2 number of the currently installed software
        3 Version number of the currently installed software
        """
        info = self.query(b"\x1BA0000\r")
        return info

    def output_position_display_value(self):
        position = self.query(b"\x1BA0100\r")
        return position

    def output_current_position(self):
        position = self.query(b"\x1BA0200\r")
        return position

    def output_error_message(self):
        error = self.query(b"\x1BA0301\r")
        return error

    def output_software_ID_number(self):
        software_id = self.query(b"\x1BA0400\r")
        return software_id

    def output_status_bar(self):
        """
        Return: a b c d e f g h
        a: Operating mode
            0 = Actual Value
            1 = Distance- To-Go
        b: Display mode for axis and coupled axes
            0 = X1
            1 = X2
            2 = X1 + X2
            3 = X1 - X2
            4 = f(X1, X2)
        c: Scale factor
            0 = Not active
            1 = Active
        d: Compensation
            0 = No compensation
            1 = Error compensation or axis-error compensation is active
        e: Stopwatch
            0 = Stopped
            1 = Stopwatch is running
        f: Unit of measure
            0 = mm
            1 = inches
            2 = DEG
            3 = DMS
            4 = rad
        g: Datum
            1 = Datum1
            2 = Datum 2
        h: Soft-key page
            1 = Page 1
            2 = Page 2
            3 = Page 3
            4 = Keyboard is locked
        """
        bar = self.query(b"\x1BA0800\r")
        return bar

    def output_status_indicators(self):
        indicator = self.query(b"\x1BA0900\r")
        return indicator

    def toggle_REF_function(self):
        indicator = self.query(b"\x1BF0000\r")
        return indicator

    def start_measurement_series_SPC(self):
        indicator = self.query(b"\x1BF0100\r")
        return indicator

    def Print(self):
        indicator = self.query(b"\x1BF0200\r")
        return indicator

    def reset_position_display(self):
        indicator = self.query(b"\x1BS0000\r")
        return indicator

    def lock_keypad(self):
        indicator = self.query(b"\x1BS0001\r")
        return indicator

    def release_keypad(self):
        indicator = self.query(b"\x1BS0002\r")
        return indicator



import time
from ..SCPI import scpi

# main class
# ==========

delay_time = 0.1

class FSW0000(scpi.scpi_family):
    manufacturer = 'Phase Matrix'
    product_name = 'QuickSyn FSW Series'
    classification = 'Signal Generator'

    _scpi_enable = '*IDN? *RCL *RST'

    def _error_check(self):
        return

    def freq_set(self, freq, unit='GHz'):
        """
        FREQ : Set CW frequency
        -----------------------
        This command sets the signal generator output frequency for the CW
        frequency mode.

        Args
        ====
        < freq : float :  >
            A frequency value.

        < unit : str : 'GHz','MHz','kHz','Hz' >
            Specify the units of the <freq>.
            'GHz', 'MHz', 'kHz' or 'Hz'. default = 'GHz'

        Returnes
        ========
        Nothing.

        Examples
        ========
        >>> s.freq_set(1, 'GHz')
        >>> s.freq_set(1.234, 'MHz')
        >>> s.freq_set(98765.4321, 'kHz')
        """
        self.com.send('FREQ {0:10f}{1}\n'.format(freq, unit))
        return

    def freq_query(self):
        """
        FREQ? : Query CW frequency
        --------------------------
        This command query the signal generator output frequency for the CW
        frequency mode.

        Args
        ====
        Nothing.

        Returnes
        ========
        < freq : float :  >
            A frequency value in Hz.

        Examples
        ========
        >>> s.freq_get()
        +2.0000000e+10
        """
        self.com.send('FREQ?\n')
        time.sleep(delay_time)
        ret = self.com.recv()
        ret = float(ret)
        ret = ret / 1000.
        return ret

    def power_set(self, pow, unit='dBm'):
        """
        POW : Set RF output power
        -------------------------
        This command sets the RF output power.

        Args
        ====
        < pow : float :  >
            A output power value.

        < unit : str : 'dBm'>
            Specify the units of the <pow>.
            'dBm'. default = 'dBm'

        Returnes
        ========
        Nothing.

        Examples
        ========
        >>> s.power_set(-130)
        >>> s.power_set(-10.2, 'dBm')
        """
        self.com.send('POW {0:f} {1}\n'.format(pow, unit))
        return

    def power_query(self):
        """
        POW? : Query RF output power
        ----------------------------
        This command query the RF output power.

        Args
        ====
        Nothing.

        Returnes
        ========
        < power : float :  >
            A power value in dBm.

        Examples
        ========
        >>> s.freq_get()
        10.1
        """
        self.com.send('POW?\n')
        time.sleep(delay_time)
        ret = self.com.recv()
        ret = float(ret)
        return ret

    def output_set(self, output):
        """
        OUTP : Enable/disable RF output
        -------------------------------
        This command enables or disables the RF output. Although you can
        configure and engage various modulations, no signal is available at
        the RF OUTPUT connector until this command is executed.

        Args
        ====
        < output : str,int : 'ON','OFF',1,0 >
            Enable/disable the RF output.

        Returnes
        ========
        Nothing.

        Examples
        ========
        >>> s.output_set('ON')
        >>> s.output_set(1)
        >>> s.output_set('OFF')
        >>> s.output_set(0)
        """
        self.com.send('OUTP:STAT {0}\n'.format(str(output)))
        return

    def output_on(self):
        """
        (Helper method) OUTP : Enable the RF output
        -------------------------------------------
        This command enables the RF output.

        Args
        ====
        Nothing.

        Returnes
        ========
        Nothing.

        Examples
        ========
        >>> s.output_on()
        """
        self.com.send('OUTP:STAT ON\n')
        return

    def output_off(self):
        """
        (Helper method) OUTP : Disable the RF output
        --------------------------------------------
        This command disables the RF output.

        Args
        ====
        Nothing.

        Returnes
        ========
        Nothing.

        Examples
        ========
        >>> s.output_off()
        """
        self.com.send('OUTP:STAT OFF\n')
        return

    def output_query(self):
        """
        OUTP? : Query RF output status
        ------------------------------
        This command queriesthe RF output status.

        Args
        ====
        Nothing.

        Returnes
        ========
        < output : int : 1,0 >
            Enable/disable the RF output.
            1 = ON, 0 = OFF


        Examples
        ========
        >>> s.output_query()
        1
        """
        self.com.send('OUTP:STAT?\n')
        time.sleep(delay_time)
        ret = self.com.readline()
        if ret.upper().find('ON') != -1: ret = 1
        elif ret.upper().find('OFF') != -1: ret = 0
        else: ret = -1
        return ret

    def use_internal_reference_source(self):
        self.com.send('ROSC:SOUR INT')
        return

    def use_external_reference_source(self):
        self.com.send('ROSC:SOUR EXT')
        return

    def reference_source_query(self):
        self.com.send('ROSC:SOUR?\n')
        time.sleep(delay_time)
        ret = self.com.readline().strip()
        return ret

    def close(self):
        self.com.close()


class FSW0010(FSW0000):
    product_name = 'FSW-0010'

class FSW0020(FSW0000):
    product_name = 'FSW-0020'



# coding:utf_8
import time
import datetime
import numpy
from ..SCPI import scpi

# main class
# ==========

class N9938A(scpi.scpi_family):
    manufacturer = 'Agilent'
    product_name = 'N9938'
    classification = 'Spectrum Analyzer'

    _scpi_enable = '*CLS *ESE *ESR? *IDN? *OPC *OPC? *OPT? *RST *WAI'


    def _error_check(self):
        err_num, err_msg = self.error_query()
        error_handler.check(err_num, err_msg)
        return

    def error_query(self):
        """
        SYST:ERR? : Query System Error
        ------------------------------
        Use this command to read the system error information.

        Args
        ====
        Nothing.

        Returns
        =======
        < err_num : int :  >
            Error number. 0 = 'No Error'

        < err_msg : str :  >
            Error message.

        Examples
        ========
        >>> p.error_query()
        (0, 'No error.')
        """
        self.com.send('SYST:ERR?')
        ret = self.com.readline()
        ret = ret.strip().split(',')
        err_num = int(ret[0])
        err_msg = ret[1].strip('"')
        return err_num, err_msg

    def installed_option_query(self):##timeout
        """
        SYST:OPT? : Installed Option Query
        ----------------------------------
        This command returns a list of the options that are installed.

        Args
        ====
        Nothing.

        Returns
        =======
        < options : list(str) :  >
            A list of the options that are installed.

        Examples
        ========
        >>> s.installed_option_query()
        ['']
        """
        #self.com.send('SYST:OPT?')
        self.com.send('*OPT?')
        ret = self.com.readline()
        self._error_check()
        ret = ret.strip().split(',')
        return ret

    def system_time_set(self, h, m, s):
        """
        SYST:TIME : Set System Time
        ---------------------------
        Sets the system time of the instrument with <"hhmmss">.
        Hour must be an integer 0 to 23.
        Minute must be an integer 0 to 59.
        Second must be an integer 0 to 59.

        Args
        ====
        < h : int : 0-23 >
            Hour to set.

        < m : int : 0-23 >
            Minute to set.

        < s : int : 0-23 >
            Second to set.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.system_time_set(12, 10, 15)
        """
        settime = '%02d%02d%02d'%(h, m, s)
        self.com.send('SYST:TIME "%s"'%(settime))
        self._error_check()
        return

    def system_time_query(self):
        """
        SYST:TIME? : Query System Time
        ------------------------------
        Query the system time of the instrument.

        Args
        ====
        Nothing.

        Returns
        =======
        < h : int : 0-23 >
            Hours.

        < m : int : 0-23 >
            Minutes.

        < s : int : 0-23 >
            Seconds.

        Examples
        ========
        >>> s.system_time_query()
        (12, 10, 15)
        """
        self.com.send('SYST:TIME?')
        ret = self.com.readline()
        self._error_check()
        ret = ret.split(',')
        h = int(ret[0])
        m = int(ret[1])
        s = int(ret[2])
        return h, m, s

    def system_date_set(self, y, m, d):
        """
        SYST:DATE : Set System Date
        ---------------------------
        Sets the system date of the real-time clock of the instrument.
        Year is a 4-digit integer. Month is an integer 1 to 12.
        Day is an integer 1 to 31 (depending on the month)

        Args
        ====
        < y : int : 0-9999 >
            Year to set.

        < m : int : 1-12 >
            Month to set.

        < d : int : 1-31 >
            Day to set.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.system_date_set(2014, 12, 23)
        """
        settime = '%04d%02d%02d'%(y, m, d)
        self.com.send('SYST:DATE "%s"'%(settime))
        self._error_check()
        return

    def system_date_query(self):
        """
        SYST:DATE? : Query System Date
        ------------------------------
        Query the system date of the instrument.

        Args
        ====
        Nothing.

        Returns
        =======
        < y : int : 0-9999 >
            Year to set.

        < m : int : 1-12 >
            Month to set.

        < d : int : 1-31 >
            Day to set.

        Examples
        ========
        >>> s.system_date_query()
        (2014, 12, 23)
        """
        self.com.send('SYST:DATE?')
        ret = self.com.readline()
        self._error_check()
        ret = ret.split(',')
        y = int(ret[0])
        m = int(ret[1])
        d = int(ret[2])
        return y, m, d

    def system_datetime_now(self):
        """
        SYST:TIME/DATE : (Helper Method) Set System DateTime Now
        --------------------------------------------------------
        Set the date and time now.

        Args
        ====
        Nothing.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.system_datetime_now()
        """
        Y = int(time.strftime('%Y'))
        m = int(time.strftime('%m'))
        d = int(time.strftime('%d'))
        self.system_date_set(Y, m, d)
        H = int(time.strftime('%H'))
        M = int(time.strftime('%M'))
        S = int(time.strftime('%S'))
        self.system_time_set(H, M, S)
        return

    def system_datetime_get(self):
        """
        SYST:TIME/DATE : (Helper Method) Get System DateTime Now
        --------------------------------------------------------
        Get the date and time.

        Args
        ====
        Nothing.

        Returns
        =======
        < datetime : datetime-object :  >
            System datetime.

        Examples
        ========
        >>> s.system_datetime_now()
        """
        Y, m, d = self.system_date_query()
        H, M, S = self.system_time_query()
        fmt = '%Y/%m/%d %H:%M:%S'
        ret = '%d/%d/%d %d:%d:%d'%(Y, m, d, H, M, S)
        timestamp = datetime.datetime.strptime(ret, fmt)
        return timestamp

    def trace_data_query(self, ch=1):
        """
        TRAC? : Query Trace Data
        ------------------------
        This query command returns the current displayed data.

        Args
        ====
        < ch : int : 1-4 >

        Returns
        =======
        < data : list(float) :  >
            A list of the data that are displayed.

        Examples
        ========
        >>> s.trace_data_query()
        array([  4.156894, -52.79362, ... , -43.07858, -41.12521 ])
        """
        self.com.send('TRAC%d:DATA?'%(ch))
        ret = self.com.readline()
        self._error_check()
        ret = numpy.array(ret.strip().split(','), float)
        return ret

    def frequency_center_set(self, freq, unit='GHz'):
        """
        FREQ:CENT : Set Center Frequency
        --------------------------------
        Set the center frequency of the spectrum analyzer.

        Args
        ====
        < freq : str or float :  >
            The center frequency to set.
            In str : '3.5 GHz', '200MHz'
            In float : 3.5, 200
                       NOTE: <unit> is referenced as the unit.

        < unit : str :  >
            The unit of the specified <freq>.
            <unit> is ignored when <freq> is specified with str.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.frequency_center_set(10)
        >>> s.frequency_center_set(10, 'MHz')
        >>> s.frequency_center_set('3.2GHz')
        """
        freq = freq_value(freq, unit)
        self.com.send('FREQ:CENT %s'%(freq.query))
        self._error_check()
        return

    def frequency_center_query(self):
        """
        FREQ:CENT? : Query Center Frequency
        -----------------------------------
        Query the center frequency of the spectrum analyzer.

        Args
        ====
        Nothing.

        Returns
        =======
        < freq : float : (Hz) >
            The center frequency. unit is in Hz.

        Examples
        ========
        >>> s.frequency_center_query()
        3500000000.0
        """
        self.com.send('FREQ:CENT?')
        ret = self.com.readline()
        self._error_check()
        ret = float(ret)
        return ret

    def frequency_center_ch_set(self, ch):
        """
        FREQ:CENT:CHAN : Set Center Frequency Channel
        ---------------------------------------------
        Set the center frequency channel of the spectrum analyzer.

        Args
        ====
        < ch : int :  >
            The center frequency channel to set.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.frequency_center_ch_set(200)
        """
        #元のコマンド self.com.send('FREQ:CENT:CHAN %d'%(ch))
        self.com.send('RAD:CHAN:CENT %d'%(ch))
        #RAD:CHAN:CENT 10
        self._error_check()
        return

    def frequency_center_ch_query(self):#?
        """
        FREQ:CENT:CHAN : Query Center Frequency Channel
        -----------------------------------------------
        Query the center frequency channel of the spectrum analyzer.

        Args
        ====
        Nothing.

        Returns
        =======
        < ch : int :  >
            The center frequency channel.

        Examples
        ========
        >>> s.frequency_center_ch_query()
        10
        """
        #元のコマンド self.com.send('FREQ:CENT:CHAN %d'%(ch))
        self.com.send('RAD:CHAN:CENT?')
        ret = self.com.readline()
        self._error_check()
        print(ret)
        ret = float(ret.lower())
        ret = int(ret)
        return ret

    def frequency_start_set(self, freq, unit='GHz'):
        """
        FREQ:STAR : Set Start Frequency
        -------------------------------
        Set the start frequency of the spectrum analyzer.

        Args
        ====
        < freq : str or float :  >
            The start frequency to set.
            In str : '3.5 GHz', '200MHz'
            In float : 3.5, 200
                       NOTE: <unit> is referenced as the unit.

        < unit : str :  >
            The unit of the specified <freq>.
            <unit> is ignored when <freq> is specified with str.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.frequency_start_set(10)
        >>> s.frequency_start_set(10, 'MHz')
        >>> s.frequency_start_set('3.2GHz')
        """
        freq = freq_value(freq, unit)
        self.com.send('FREQ:STAR %s'%(freq.query))
        #FREQ:STAR 10E6
        self._error_check()
        return

    def frequency_start_query(self):
        """
        FREQ:STAR? : Query Start Frequency
        ----------------------------------
        Query the start frequency of the spectrum analyzer.

        Args
        ====
        Nothing.

        Returns
        =======
        < freq : float : (Hz) >
            The start frequency. unit is in Hz.

        Examples
        ========
        >>> s.frequency_start_query()
        0.0
        """
        self.com.send('FREQ:STAR?')
        ret = self.com.readline()
        self._error_check()
        ret = float(ret)
        return ret

    def frequency_stop_set(self, freq, unit='GHz'):
        """
        FREQ:STOP : Set Stop Frequency
        ------------------------------
        Set the stop frequency of the spectrum analyzer.

        Args
        ====
        < freq : str or float :  >
            The stop frequency to set.
            In str : '3.5 GHz', '200MHz'
            In float : 3.5, 200
                       NOTE: <unit> is referenced as the unit.

        < unit : str :  >
            The unit of the specified <freq>.
            <unit> is ignored when <freq> is specified with str.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.frequency_stop_set(10)
        >>> s.frequency_stop_set(10, 'MHz')
        >>> s.frequency_stop_set('3.2GHz')
        """
        freq = freq_value(freq, unit)
        self.com.send('FREQ:STOP %s'%(freq.query))
        #FREQ:STOP 10E6
        self._error_check()
        return

    def frequency_stop_query(self):
        """
        FREQ:STOP? : Query Start Frequency
        ----------------------------------
        Query the stop frequency of the spectrum analyzer.

        Args
        ====
        Nothing.

        Returns
        =======
        < freq : float : (Hz) >
            The stop frequency. unit is in Hz.

        Examples
        ========
        >>> s.frequency_stop_query()
        0.0
        """
        self.com.send('FREQ:STOP?')
        ret = self.com.readline()
        self._error_check()
        ret = float(ret)
        return ret

    def frequency_span_set(self, freq, unit='GHz'):
        """
        FREQ:SPAN : Set Frequency Span
        ------------------------------
        Set the frequency span. Setting the span to 0 Hz puts the analyzer
        into zero span.

        Limited to between 10 Hz and 100 MHz

        Args
        ====
        < freq : str or float :  >
            The span frequency to set.
            In str : '3.5 GHz', '200MHz'
            In float : 3.5, 200
                       NOTE: <unit> is referenced as the unit.

        < unit : str :  >
            The unit of the specified <freq>.
            <unit> is ignored when <freq> is specified with str.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.frequency_span_set(10)
        >>> s.frequency_span_set(10, 'MHz')
        >>> s.frequency_span_set('3.2GHz')
        """
        freq = freq_value(freq, unit)
        self.com.send('FREQ:SPAN %s'%(freq.query))
        #FREQ:SPAN 10E6
        self._error_check()
        return

    def frequency_span_query(self):
        """
        FREQ:SPAN? : Query Span Frequency
        ---------------------------------
        Query the span frequency of the spectrum analyzer.

        Args
        ====
        Nothing.

        Returns
        =======
        < freq : float : (Hz) >
            The span frequency. unit is in Hz.

        Examples
        ========
        >>> s.frequency_span_query()
        0.0
        """
        self.com.send('FREQ:SPAN?')
        ret = self.com.readline()
        self._error_check()
        ret = float(ret)
        return ret

    def reference_level_set(self, level):
        """
        DISP:WIND:TRAC:Y:RLEV : Set Refelence Level
        -------------------------------------------
        This command sets the reference level for the Y-axis.

        Args
        ====
        < level : float :  >
            The reference level to set. unit is in dBm.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.reference_level_set(0)
        >>> s.reference_level_set(-20)
        """
        self.com.send('DISP:WIND:TRAC:Y:RLEV %f dBm'%(level))
        #DISP:WIND:TRAC1:Y:RLEV -50
        self._error_check()
        return

    def reference_level_query(self):
        """
        DISP:WIND:TRAC:Y:RLEV? : Query Refelence Level
        ----------------------------------------------
        Query the reference level for the Y-axis.

        Args
        ====
        Nothing.

        Returns
        =======
        < level : float :  >
            The reference level. unit is in dBm.

        Examples
        ========
        >>> s.reference_level_query()
        0.0
        """
        self.com.send('DISP:WIND:TRAC:Y:RLEV?')
        ret = self.com.readline()
        self._error_check()
        ret = float(ret)
        return ret

    def attenuation_set(self, att):
        """
        POW:ATT : Set Input Attenuation
        -------------------------------
        Set the input attenuator of the spectrum analyzer.

        Args
        ====
        < att : int : 0, 50, 5step >
            The input attenuation to set. unit is in dB.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.attenuation_set(10)
        """
        self.com.send('POW:ATT %f dB'%(att))
        #POW:ATT 30
        self._error_check()
        return

    def attenuation_query(self):
        """
        POW:ATT? : Query Input Attenuation
        -----------------------------------
        Query the input attenuatior level.

        Args
        ====
        Nothing.

        Returns
        =======
        < att : float :  >
            The input attenuator level. unit is in dB.

        Examples
        ========
        >>> s.attenuation_query()
        0.0
        """
        self.com.send('POW:ATT?')
        ret = self.com.readline()
        self._error_check()
        ret = int(ret)
        return ret

    def attenuation_auto_set(self, auto):
        """
        POW:ATT:AUTO : Set Input Attenuation AUTO/MANUAL
        ------------------------------------------------
        Set the input attenuator state to AUTO/MANUAL.

        Args
        ====
        < auto : int or str : 0,1,'OFF','ON' >
            The input attenuation state to set.
            0 = 'OFF', 1 = 'ON'

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.attenuation_auto_set(0)
        >>> s.attenuation_auto_set('ON')
        """
        self.com.send('POW:ATT:AUTO %s'%(str(auto)))
        #POW:ATT:AUTO 1
        self._error_check()
        return

    def attenuation_auto_query(self):
        """
        POW:ATT:AUTO? : Query Input Attenuation AUTO/MANUAL
        ---------------------------------------------------
        Query the input attenuator state.

        Args
        ====
        Nothing.

        Returns
        =======
        < auto : int : 0,1  >
            The input attenuation state.
            0 = 'OFF', 1 = 'ON'

        Examples
        ========
        >>> s.attenuation_auto_query()
        0
        """
        self.com.send('POW:ATT:AUTO?')
        ret = self.com.readline()
        self._error_check()
        ret = int(ret)
        return ret

    def scalediv_set(self, div):
        """
        DISP:WIND:TRAC:Y:PDIV : Set Scale/DIV
        -------------------------------------
        This command sets the per-division display scaling for the y-axis
        when scale type of Y axis is set to Log.

        Args
        ====
        < div : int : 1,2,5,10 >
            The per-division display scaling to set. unit is in dB.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.scalediv_set(10)
        """
        self.com.send('DISP:WIND:TRAC:Y:PDIV DIV%d'%(div))
        #DISP:WIND:TRAC1:Y:PDIV -.25
        self._error_check()
        return

    def scalediv_query(self):
        """
        DISP:WIND:TRAC:Y:PDIV? : Query Scale/DIV
        ----------------------------------------
        Query display scaling.

        Args
        ====
        Nothing.

        Returns
        =======
        < div : int : 1,2,5,10 >
            The per-division display scaling. unit is in dB.

        Examples
        ========
        >>> s.scalediv_query()
        10
        """
        self.com.send('DISP:WIND:TRAC:Y:PDIV?')
        ret = self.com.readline()
        self._error_check()
        ret = ret.lower()
        ret = int(float(ret))

        return ret

    def scaletype_set(self, stype):
        """
        DISP:WIND:TRAC:Y:SPAC : Set Scale Type
        --------------------------------------
        Toggles the vertical graticule divisions between logarithmic unit and
        linear unit. The default logarithmic unit is dBm, and the linear unit
        is mV.

        Args
        ====
        < stype : str : 'LOG','LIN' >
            The unit to set.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.scaletype_set('LOG')
        """
        #N9342 self.com.send('DISP:WIND:TRAC:Y:SPAC %s'%(stype))
        self.com.send('AMPL:SCAL %s'%(stype))
        #AMPL:SCAL LIN
        self._error_check()
        return

    def scaletype_query(self):
        """
        DISP:WIND:TRAC:Y:SPAC? : Query Scale Type
        -----------------------------------------
        Query the scale type.

        Args
        ====
        Nothing.

        Returns
        =======
        < stype : str : 'LOG','LIN' >
            The scale type.

        Examples
        ========
        >>> s.scaletype_query()
        """
        #N9342 self.com.send('DISP:WIND:TRAC:Y:SPAC?')
        self.com.send('AMPL:SCAL?')
        #AMPL:SCAL?
        ret = self.com.readline().strip()
        self._error_check()
        return ret

    def resolution_bw_set(self, bw, unit='MHz'):
        """
        BAND : Set Resolution Bandwidth(RBW)

        -------------------------------
        Specifies the resolution bandwidth. For numeric entries, all RBW types
        choose the nearest (arithmetically, on a linear scale, rounding up)
        available RBW to the value entered.

        Args
        ====
        < bw : str or float : 10Hz-3MHz >
            The bandwidth to set.
            In str : '3 MHz', '10kHz'
            In float : 10, 3
                       NOTE: <unit> is referenced as the unit.

        < unit : str :  >
            The unit of the specified <freq>.
            <unit> is ignored when <freq> is specified with str.



         Zero span:       10Hz to 5MHz   1,3,10 sequence
         Non-zero span:   1Hz to 5MHz
                          1,1.5,2,3,5,7.5,10 sequence <300kHz
                          300kHz,1MHz,3Mhz,5MHz
                          (Other RBWs may be set depending on settings)
                          Step key change RBW in 1,3,10 sequence

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.resolution_bw_set(3, 'MHz')
        >>> s.resolution_bw_set('100Hz')
        """
        freq = freq_value(bw, unit)
        print(freq.query)
        print(bw,unit)

        self.com.send('BAND %s'%(freq.query))
        self._error_check()
        return

    def resolution_bw_query(self):
        """
        BAND? : Query Resolution Bandwidth
        ----------------------------------
        Query resolution bandwidth.

        Args
        ====
        Nothing.

        Returns
        =======
        < bw : float :  >
            The bandwidth. unit is Hz.

        Examples
        ========
        >>> s.resolution_bw_query()
        3000000.0
        """
        self.com.send('BAND?')
        ret = self.com.readline()
        self._error_check()
        ret = float(ret)
        return ret

    def resolution_bw_auto_set(self, auto):
        """
        BAND:AUTO : Set Resolution Bandwidth AUTO/MANUAL
        ------------------------------------------------
        This command turns on/off auto resolution bandwidth state.

        Args
        ====
        < auto : int or str : 0,1,'OFF','ON' >
            The resolution BW state to set.
            0 = 'OFF', 1 = 'ON'

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.resolution_bw_auto_set(0)
        >>> s.resolution_bw_auto_set('ON')
        """
        self.com.send('BAND:AUTO %s'%(str(auto)))
        self._error_check()
        return

    def resolution_bw_auto_query(self):
        """
        BAND:AUTO? : Query Resolution Bandwidth AUTO/MANUAL
        ---------------------------------------------------
        Query the AUTO status of the resolution BW.

        Args
        ====
        Nothing.

        Returns
        =======
        < auto : int or str : 0,1,'OFF','ON' >
            The resolution BW state.
            0 = 'OFF', 1 = 'ON'

        Examples
        ========
        >>> s.resolution_bw_auto_query()
        1
        """
        self.com.send('BAND:AUTO?')
        ret = self.com.readline()
        self._error_check()
        ret = int(ret)
        return ret

    def video_bw_set(self, bw, unit='MHz'):
        """
        BAND:VID : Set Video Bandwidth
        ------------------------------
        Specifies the video bandwidth.

        Args
        ====
        < bw : str or float : 10Hz-3MHz >
            The bandwidth to set.
            In str : '3 MHz', '10kHz'
            In float : 10, 3
                       NOTE: <unit> is referenced as the unit.

        < unit : str : 'MHz','kHz','Hz'>
            The unit of the specified <freq>.
            <unit> is ignored when <freq> is specified with str.


        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.video_bw_set(3, 'MHz')
        >>> s.video_bw_set('100Hz')
        """
        freq = freq_value(bw, unit)
        self.com.send('BAND:VID %s'%(freq.query))
        self._error_check()
        return

    def video_bw_query(self):
        """
        BAND:VID? : Query Video Bandwidth
        ---------------------------------
        Query the video bandwidth.

        1Hz to 5MHz  1,1.5,2,3,5,7.5,10 sequence

        Args
        ====
        Nothing.

        Returns
        =======
        < bw : float :  >
            The bandwidth. unit is Hz.

        Examples
        ========
        >>> s.video_bw_query()
        """
        self.com.send('BAND:VID?')
        ret = self.com.readline()
        self._error_check()
        ret = float(ret)
        return ret

    def video_bw_auto_set(self, auto):
        """
        BAND:VID:AUTO : Set Video Bandwidth AUTO/MANUAL
        -----------------------------------------------
        This command turns on/off auto video bandwidth state.

        Args
        ====
        < auto : int or str : 0,1,'OFF','ON' >
            The resolution BW state to set.
            0 = 'OFF', 1 = 'ON'

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.video_bw_auto_set(0)
        >>> s.video_bw_auto_set('ON')
        """
        self.com.send('BAND:VID:AUTO %s'%(str(auto)))
        self._error_check()
        return

    def video_bw_auto_query(self):
        """
        BAND:AUTO? : Query Video Bandwidth AUTO/MANUAL
        ----------------------------------------------
        Query the AUTO status of the Video BW.

        Args
        ====
        Nothing.

        Returns
        =======
        < auto : int or str : 0,1,'OFF','ON' >
            The resolution BW state.
            0 = 'OFF', 1 = 'ON'

        Examples
        ========
        >>> s.resolution_bw_auto_query()
        1
        """
        self.com.send('BAND:VID:AUTO?')
        ret = self.com.readline()
        self._error_check()
        ret = int(ret)
        return ret

    def average_set(self, num, ch=1):
        """
        AVER:TRACn:COUN : Set Average Number
        -------------------------------
        Specifies the number of measurements that are combined.

        Args
        ====
        < num : int : 1-8192 >
            The number count of measurements that are combined.

        < ch : int : 1-4 >
            The number of a trace to set average.
            default = 1

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.average_set(1)
        >>> s.average_set(11, ch=2)
        """
        #N9342 self.com.send('AVER:TRAC%d:COUN %d'%(ch, num))
        self.com.send('AVER:COUN %d'%(ch,num))
        self._error_check()
        return

    def average_query(self, ch=1):
        """
        AVER:TRACn:COUN? : Query Average Number
        ---------------------------------------
        Query the average number.

        Args
        ====
        < ch : int : 1-4 >
            The number of a trace to query average.
            default = 1

        Returns
        =======
        < num : int : 1-8192 >
            The number count of measurements that are combined.

        Examples
        ========
        >>> s.average_query()
        1

        >>> s.average_query(ch=2)
        11
        """
        #N9342 self.com.send('AVER:TRAC%d:COUN?'%(ch))
        self.com.send('AVER:COUN?')
        ret = self.com.readline()
        self._error_check()
        ret = int(ret)
        return ret

    def average_onoff_set(self, on_off, ch=1):
        """
        AVER:TRACn : Set Average ON/OFF
        -------------------------------
        Sets the ON/OFF of average process.

        Args
        ====
        < on_off : int or str : 0,1,'OFF','ON' >
            The average ON/OFF state.
            0 = 'OFF', 1 = 'ON'

        < ch : int : 1-4 >
            The number of a trace to set average.
            default = 1

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.average_onoff_set(1)
        >>> s.average_onoff_set('OFF', ch=2)
        """
        self.com.send('AVER:TRAC%d %s'%(ch, str(on_off)))
        self._error_check()
        return

    def average_onoff_query(self, ch=1):
        """
        AVER:TRACn? : Query Average ON/OFF
        ----------------------------------
        Query the ON/OFF status of average process.

        Args
        ====
        < ch : int : 1-4 >
            The number of a trace to set average.
            default = 1

        Returns
        =======
        < on_off : int or str : 0,1,'OFF','ON' >
            The average ON/OFF state.
            0 = 'OFF', 1 = 'ON'

        Examples
        ========
        >>> s.average_onoff_query()
        0

        >>> s.average_onoff_query(ch=2)
        1
        """
        #N9342 self.com.send('AVER:TRAC%d?'%(ch))
        self.com.send('SYST:GPS:AVER%d?'%(ch))
        ret = self.com.readline()
        self._error_check()
        ret = int(ret)
        return ret

    def average_restart(self, ch=1):
        """
        AVER:TRACn:CLE : Restart Average
        --------------------------------
        Restart the average process.

        Args
        ====
        < ch : int : 1-4 >
            The number of a trace to set average.
            default = 1

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.average_restart()
        >>> s.average_restart(ch=2)
        """
        #N9342 self.com.send('AVER:TRAC%d:CLE'%(ch))
        self.com.send('INIT:REST%d;*OPC?'%(ch))
        ret = self.com.readline()
        #INIT:REST;*OPC?
        self._error_check()
        return ret

    def sweep_time_set(self, sweep):
        """
        SWE:TIME : Set Sweep Time
        -------------------------
        Specifies the time in which the instrument sweeps the display.
        A span value of 0 Hz causes the analyzer to enter zero span mode.
        In zero span the X-axis represents time rather than frequency.

        Args
        ====
        < sweep : float :  >
            The sweep time. unit is sec.

        Returns
        =======
        Nothing.

        Examples
        ========
        >>> s.sweep_time_set(5)
        """
        self.com.send('SWE:TIME %fs'%(sweep))
        self._error_check()
        return

    def sweep_time_query(self):
        """
        SWE:TIME? : Query Sweep Time
        ----------------------------
        Query sweep time.

        Args
        ====
        Nothing.

        Returns
        =======
        < sweep : float :  >
            The sweep time. unit is sec.

        Examples
        ========
        >>> s.sweep_time_query()
        0.287
        """
        self.com.send('SWE:TIME?')
        ret = self.com.readline()
        self._error_check()
        ret = float(ret)
        return ret

    def gen_xaxis(self):
        start = self.frequency_start_query()
        stop = self.frequency_stop_query()
        num = len(self.trace_data_query())
        xaxis = numpy.linspace(start, stop, num)
        return xaxis






# ==============
# Helper Classes
# ==============

# Values
# ======
class freq_value(object):
    def __init__(self, value, unit):
        if type(value) == str:
            self.unit = value.strip('1234567890. ')
            self.value = float(value.strip(self.unit+' '))
        else:
            self.unit = unit.strip()
            self.value = float(value)
            pass
        self.query = '%.10f %s'%(self.value, self.unit)
        pass


# Error Class
# ===========
class error_item(object):
    num = 0
    msg = ''
    txt = ''

    def __init__(self, num, msg, txt):
        self.num = num
        self.msg = msg
        self.txt = txt
        pass

class error_handler(object):
    error_list = [
        error_item(0, 'No error', ''),
        error_item(-410, 'Query INTERRUPTED', 'Indicates that a condition causing an INTERRUPTED query occurred (see IEEE 488.2, 6.3.2.7)'),
        error_item(-350, 'Query overflow', 'Indicates the SCPI remote interface error queue overflowed.'),
        error_item(-321, 'Out of memory', 'Indicates an internal operation needed more memory than that was available.'),
        error_item(-224, 'Illegal parameter value', 'Indicates you sent a parameter for this command that is NOT allowed.'),
        error_item(-223, 'Too much data', 'Indicates a legal program data element of block, expression or string type was received that contained more data than the device could handle due to related device-specific requirements or memory.'),
        error_item(-222, 'Data out of range', 'Indicates a legal data was parsed but could not be executed because of the interpreted value was outside the legal range defined by the analyzer. The displayed results may be clipped.'),
        error_item(-220, 'No matched module', 'Indicates no matched measurement or mode found.'),
        error_item(-200, 'Execution error', 'This is a generic execution error for devices that cannot detect more specific errors. The code indicates on those execution errors defined in IEEE 488.2, 11.5.1.1.4 has occurred.'),
        error_item(-171, 'Invalid expression', 'Indicates the data element was invalid, for example, unmatched parentheses, or an illegal character.'),
        error_item(-144, 'Character data long', 'Indicates the character data contained more than 12 characters. (see IEEE 488.2, 7.7.1.4)'),
        error_item(764, 'Unable to save file', 'Indicates a failure occurred while saving a file. The file was not saved.'),
        error_item(762, 'Unable to load file', 'Indicates a failure occurred while loading a file. The file was not loaded.'),
        error_item(612, 'File NOT found', 'Indicates the analyzer could not find the specified file.'),
        error_item(173, 'Lame package, please upgrade firmware', 'Indicates the current firmware is not mostly updated. The firmware needs upgrading.'),
        error_item(172, 'Option install fail, invalid option licence', 'Indicates the option could not be installed, because of the invalid option licence.'),
        error_item(171, 'File loaded', 'Indicates the file loading succeeded.'),
        error_item(170, 'The menu is unavailable in this case', 'Indicates the memo is invalid in this case.'),
        error_item(166, 'Marker counter opened in fast sweep', 'Indicates the counteris accuracy decreases when in fast sweep mode.'),
        error_item(162, 'Cannot open fast sweep in this case', 'Indicates current settings do not allow youtoopenfastsweep.Forexample, currently in FFT, zero span, or any measurement in power suite does not allow fast sweep.'),
        error_item(156, 'Incorrect alignment file on flash', 'Indicates an invalid alignment file.'),
        error_item(153, 'RF EEPROM operate fail', 'Indicates an error when writing alignment file.'),
        error_item(152, 'Alignment file oversize', 'Indicates the alignment file has error or invalid data.'),
        error_item(151, 'DSP boot fail', 'Indicates the analyzer cannot process measurement currently, wait the analyzer to reboot. If analyzer froze, try to restart the analyzer.'),
        error_item(150, 'Mixer overload', 'Indicates the first mixer in danger. Either increase input attenuation or decrease the input signal level.'),
        error_item(149, 'Mixer saturate', 'Indicates you need to either increase the input attenuation or decrease the input signal level.'),
        error_item(147, 'Incorrect alignment data in EEPROM', 'Indicates invalid alignment data occurred.'),
        error_item(143, 'Final IF overload', 'Indicates you need to either increase the input attenuation or decrease the input signal level.'),
        error_item(138, 'USB device NOT ready', 'Indicates the USB device is not detected.'),
        error_item(136, 'RBW limit to 30 kHz when in fast sweep', 'Indicates the analyzer automatically couple the RBW to 30 kHz when in fast sweep mode.'),
        error_item(130, 'Meas uncal', 'Indicates the measurement is uncalibrated due to fast sweeping through a narrow RBW filter. Check the sweep time, span, and bandwidth settings, or use auto coupling.'),
        error_item(119, 'RF Board Changed', 'Indicates the RF board was changed, the analyzer needs re-load the alignment data.'),
        error_item(116, 'Cannotcommunicate with RF', 'IndicatestheMCUcannotfindtheRF board.'),
    ]

    @classmethod
    def check(cls, num, msg):
        if num==0: return
        for e in cls.error_list:
            if msg == e.msg:
                emsg = '%s (%d)'%(e.msg, e.num)
                msg = 'Power meter returned Error message.\n'
                msg += '*'*len(emsg) + '\n'
                msg += emsg + '\n'
                msg += '*'*len(emsg) + '\n'
                msg += e.txt + '\n'
                raise Exception(msg)
            continue
        _msg = 'Power meter returned Error message.\n'
        emsg = '%s (%d)\n'%(msg, num)
        _msg += '*'*len(emsg) + '\n'
        _msg += emsg
        _msg += '*'*len(emsg) + '\n'
        raise Exception(_msg)
        return



import time
from ..SCPI import scpi

# main class
# ==========

class E4418(scpi.scpi_family):
    manufacturer = 'Agilent'
    product_name = 'E4418'
    classification = 'Power Meter'
    
    _scpi_enable = '*CLS *DDT *ESE *ESR? *IDN? *OPC *OPT? *RCL *RST *SAV ' +\
                   '*SRE *STB? *TST? *WAI'
    
    def _error_check(self):
        err_num, err_msg = self.error_query()
        error_handler.check(err_num, err_msg)
        return
    
    def error_query(self):
        """
        SYST:ERR? : Query Error Numbers
        -------------------------------
        This query returns error numbers and messages from the power meter's
        error queue. When an error is generated by the power meter, it stores
        an error number and corresponding message in the error queue. One error
        is removed from the error queue each time the SYSTem:ERRor? command is
        executed. The errors are cleared in the order of first-in first-out,
        that is, the oldest errors are cleared first. To clear all the errors
        from the error queue, execute the *CLS command. When the error queue is
        empty, subsequent SYSTem:ERRor? queries return a +0, "No error"
        message. The error queue has a maximum capacity of 30 errors.
        
        Args
        ====
        Nothing.
        
        Returns
        =======
        < err_num : int :  >
            Error number. 0 = 'No Error'
        
        < err_msg : str :  >
            Error message.
        
        Examples
        ========
        >>> p.error_query()
        (0, 'No error.')
        """
        self.com.send('SYST:ERR?')
        ret = self.com.readline()
        ret = ret.strip().split(',')
        err_num = int(ret[0])
        err_msg = ret[1].strip('"')
        return err_num, err_msg
        
    def zeroing(self, ch=1):
        """
        CALn:ZERO:AUTO : Zeroing
        ------------------------
        This command causes the power meter to perform its zeroing routine on
        the specified channel when ONCE is selected. Zeroing takes
        approximately 10 seconds. This adjusts the power meter for a zero power
        reading with no power supplied to the power sensor. The 0|OFF parameter
        is only required for the query response and is ignored in the command.
        If 1|ON is selected, it causes the error -224, "Illegal parameter
        value" to occur.
        
        The command assumes that the power sensor is not connected to a power
        source.
        
        Args
        ====
        < ch : int : 1,2 >
            Specify the channel to perform a zeroing. (1, 2)
            default = 1
        
        Returns
        =======
        Nothing.
        
        Examples
        ========
        >>> p.zeroing()
        >>> p.zeroing(ch=2)
        """
        self.com.send('CAL%d:ZERO:AUTO ONCE'%(ch))
        self._error_check()
        time.sleep(10)
        self._error_check()
        return
        
    def measure(self, ch=1, unit='DEF', resolution='DEF', wait=7):
        """
        MEASn? : Measuring
        ------------------
        This command sets the specified window's measurement function to single
        channel with relative mode off, aborts, configures the window then
        initiates channel A or B, calculates the measurement result and places
        the result on the bus.
        
        Args
        ====
        < ch : int : 1,2 >
            Specify the channel to perform a measuring. (1, 2)
            default = 1            
        
        < unit : str : 'DEF','dBm','W' >
            Specify the units of the returned power level.
            'DEF', 'dBm', or 'W'. default = 'DEF'
        
        < resolution : str, or int : 'DEF',1.0,0.1,0.01,0.001 >
            Specify the resolution of the returned value.
            default = 'DEF'
        
        Returnes
        ========
        < power : float :  >
            Measured power level. The units is specified by <unit>.
        
        Examples
        ========
        >>> p.measure()
        >>> p.measure(ch=2)
        """
        self.com.send('MEAS%d? %s, %s'%(ch, unit, resolution))
        time.sleep(wait)
        ret = self.com.readline()
        self._error_check()
        ret = float(ret.strip())
        return ret
    
    def average_on_off(self, on_off, ch=1):
        """
        SENSn:AVER : Set Average ON/OFF
        -------------------------------
        This command is used to enable and disable averaging.
        
        Args
        ====
        < on_off : int or str : 0,1,'ON','OFF' >
            Specify the averaging status.
            0 = 'OFF', 1 = 'ON'
        
        < ch : int : 1,2 >
            Specify the channel to set a averaging status. (1, 2)
            default = 1            
        
        Returnes
        ========
        Nothing.
        
        Examples
        ========
        >>> p.average_on_off(1)
        >>> p.average_on_off('OFF', ch=2)
        """
        self.com.send('SENS%d:AVER %s'%(ch, str(on_off)))
        self._error_check()
        return
        
    def average_on_off_query(self, ch=1):
        """
        SENSn:AVER? : Query Average ON/OFF
        -----------------------------------
        Check the average status.
        
        Args
        ====
        < ch : int : 1,2 >
            Specify the channel to query a averaging status. (1, 2)
            default = 1            
        
        Returnes
        ========
        < on_off : int : 1,0 >
            Average status. 1 = ON, 0 = OFF
        
        Examples
        ========
        >>> p.average_on_off_query()
        1
        
        >>> p.average_on_off_query(ch=2)
        0
        """
        self.com.send('SENS%d:AVER?'%(ch))
        ret = self.com.readline()
        self._error_check()
        ret = int(ret)
        return ret
    
    def average_count(self, count, ch=1):
        """
        SENSn:AVER:COUN : Set Average Count
        -----------------------------------
        This command is used to enter a value for the filter length. If
        [SENSe[1]]|SENSe2:AVERage:COUNt:AUTO is set to ON then entering a value
        for the filter length automatically sets it to OFF. Increasing the 
        value of filter length increases measurement accuracy but also
        increases the time taken to make a power measurement.
        
        Entering a value using this command automatically turns the
        [SENSe[1]]|SENSe2:AVERage:STATe command to ON.
        
        Args
        ====
        < count : int :  >
            Specify the count for averaging (filter length).
        
        < ch : int : 1,2 >
            Specify the channel to set a averaging count. (1, 2)
            default = 1            
        
        Returnes
        ========
        Nothing.
        
        Examples
        ========
        >>> p.average_count(128)
        >>> p.average_count(64, ch=2)
        """
        self.com.send('SENS%d:AVER:COUN %d'%(ch, count))
        self._error_check()
        return
        
    def average_count_query(self, ch=1):
        """
        SENSn:AVER:COUN? : Query Average Count
        ---------------------------------------
        Check the count for averaging.
        
        Args
        ====
        < ch : int : 1,2 >
            Specify the channel to set a averaging count. (1, 2)
            default = 1            
        
        Returnes
        ========
        < count : int :  >
            The count for averaging (filter length).
                
        Examples
        ========
        >>> p.average_count_query()
        128
        
        >>> p.average_count_query(ch=2)
        64
        """
        self.com.send('SENS%d:AVER:COUN?'%(ch))
        ret = self.com.readline()
        self._error_check()
        ret = int(ret)
        return ret
        
    
        
    
class EPM441A(E4418):
    product_name = 'EPM-441A'

class EPM442A(E4418):
    product_name = 'EPM-442A'

class E4418B(E4418):
    product_name = 'E4418B'

class E4419B(E4418):
    product_name = 'E4419B'




# ==============
# Helper Classes
# ==============

# Error Class
# ===========
class error_item(object):
    num = 0
    msg = ''
    txt = ''
    
    def __init__(self, num, msg, txt):
        self.num = num
        self.msg = msg
        self.txt = txt
        pass

class error_handler(object):
    error_list = [
        error_item(0, 'No error', ''),
        error_item(-101, 'Invalid character', 'An invalid character was found in the command string. You may have inserted a character such as #, $ or % in the command header or within a parameter. For example, LIM:LOW O#.'),
        error_item(-102, 'Syntax error', 'Invalid syntax was found in the command string. For example, LIM:CLE:AUTO, 1 or LIM:CLE:AUTO 1.'),
        error_item(-103, 'Invalid separator', 'An invalid separator was found in the command string. You may have used a comma instead of a colon, semicolon or blank space; or you may have used a blank space instead of a comma. For example, OUTP:ROSC,1.'),
        error_item(-105, 'GET not allowed', 'A Group Execute Trigger (GET) is not allowed within a command string.'),
        error_item(-108, 'Parameter not allowed', 'More parameters were received than expected for the command. You may have entered an extra parameter or added a parameter to a command that does not accept a parameter. For example, CAL 10.'),
        error_item(-109, 'Missing parameter', 'Fewer parameters were received than expected for the command. You omitted one or more parameters that are required for this command. For example, AVER:COUN.'),
        error_item(-112, 'Program mnemonic too long', 'A command header was received which contained more than the maximum 12 characters allowed. For example, SENSeAVERageCOUNt 8.'),
        error_item(-113, 'Undefined header', 'A command was received that is not valid for this power meter. You may have misspelled the command, it may not be a valid command or you may have the wrong interface selected. If you are using the short form of the command, remember that it may contain up to four letters. For example, TRIG:SOUR IMM.'),
        error_item(-121, 'Invalid character in number', 'An invalid character was found in the number specified for a parameter value. For example, SENS:AVER:COUN 128#H.'),
        error_item(-123, 'Exponent too large', 'A numeric parameter was found whose exponent was larger than 32,000. For example, SENS:COUN 1E34000.'),
        error_item(-124, 'Too many digits', 'A numeric parameter was found whose mantissa contained more than 255 digits, excluding leading zeros.'),
        error_item(-128, 'Numeric data not allowed', 'A numeric value was received within a command which does not accept a numeric value. For example, MEM:CLE 24.'),
        error_item(-131, 'Invalid suffix', 'A suffix was incorrectly specified for a numeric parameter. You may have misspelled the suffix. For example, SENS:FREQ 200KZ.'),
        error_item(-134, 'Suffix too long', 'A suffix used contained more than 12 characters. For example, SENS:FREQ 2MHZZZZZZZZZZZ.'),
        error_item(-138, 'Suffix not allowed', 'A suffix was received following a numeric parameter which does not accept a suffix. For example, INIT:CONT 0Hz.'),
        error_item(-148, 'Character data not allowed', 'A discrete parameter was received but a character string or a numeric parameter was expected. Check the list of parameters to verify that you have used a valid parameter type. For example, MEM:CLE CUSTOM_1.'),
        error_item(-151, 'Invalid string data', 'An invalid string was received. Check to see if you have enclosed the character string in single or double quotes. For example, MEM:CLE "CUSTOM_1.'),
        error_item(-158, 'String data not allowed', 'A character string was received but is not allowed for the command. Check the list of parameters to verify that you have used a valid parameter type. For example, LIM:STAT `ON\'.'),
        error_item(-161, 'Invalid block data', 'A block data element was expected but was invalid for some reason. For example, *DDT #15FET. The 5 in the string indicates that 5 characters should follow, whereas in this example there are only 3.'),
        error_item(-168, 'Block data not allowed', 'A legal block data element was encountered but not allowed by the power meter at this point. For example SYST:LANG #15FETC?.'),
        error_item(-178, 'Expression data not allowed', 'A legal expression data was encountered but not allowed by the power meter at this point. For example SYST:LANG (5+2).'),
        error_item(-211, 'Trigger ignored', 'Indicates that <GET> or *TRG or TRIG:IMM was received and recognized by the device but was ignored because the power meter was not in the wait for trigger state.'),
        error_item(-213, 'Init ignored', 'Indicates that a request for a measurement initiation was ignored as the power meter was already initiated. For example, INIT:CONT ON INIT.'),
        error_item(-214, 'Trigger deadlock', 'TRIG:SOUR was set to HOLD or BUS and a READ? or MEASure? was attempted, expecting TRIG:SOUR to be set to IMMediate.'),
        error_item(-220, 'Parameter error;Frequency list must be in ascending order.', 'Indicates that the frequencies entered using the MEMory:TABLe:FREQuency command are not in ascending order.'),
        error_item(-221, 'Settings conflict', 'This command occurs under a variety of conflicting conditions. The following list gives a few examples of where this error may occur: * If the READ? parameters do not match the current settings. * If you are in fast mode and attempting to switch on for example, averaging, duty cycle or limits. * Trying to clear a sensor calibration table when none is selected.'),
        error_item(-221, 'Settings conflict;DTR/DSR not available on RS422', 'DTR/DSR is only available on the RS232 interface.'),
        error_item(-222, 'Data out of range', 'A numeric parameter value is outside the valid range for the command. For example, SENS:FREQ 2KHZ.'),
        error_item(-224, 'Illegal parameter value', 'A discrete parameter was received which was not a valid choice for the command. You may have used an invalid parameter choice. For example, TRIG:SOUR EXT.'),
        error_item(-226, 'Lists not same length', 'This occurs when SENSe:CORRection:CSET[1]|CSET2:STATe is set to ON and the frequency and calibration/offset lists do not correspond in length.'),
        error_item(-230, 'Data corrupt or stale', 'This occurs when a FETC? is attempted and either a reset has been received or the power meter state has changed such that the current measurement is invalidated (for example, a change of frequency setting or triggering conditions).'),
        error_item(-230, 'Data corrupt or stale;Please zero and calibrate Channel A', 'When CAL[1|2]:RCAL is set to ON and the sensor currently connected to channel A has not been zeroed and calibrated, then any command which would normally return a measurement result (for example FETC?, READ? or MEAS?) will generate this error message.'),
        error_item(-230, 'Data corrupt or stale;Please zero Channel A', 'When CAL[1|2]:RCAL is set to ON and the sensor currently connected to channel A has not been zeroed, then any command which would normally return a measurement result (for example FETC?, READ? or MEAS?) will generate this error message.'),
        error_item(-230, 'Data corrupt or stale;Please calibrate Channel A', 'When CAL[1|2]:RCAL is set to ON and the sensor currently connected to channel A has not been calibrated, then any command which would normally return a measurement result (for example FETC?, READ? or MEAS?) will generate this error message'),
        error_item(-231, 'Data questionable;CAL ERROR', 'Power meter calibration failed. The most likely cause is attempting to calibrate without applying a 1 mW power to the power sensor.'),
        error_item(-231, 'Data questionable;Input Overload', 'The power input to Channel A exceeds the power sensor\'s maximum range.'),
        error_item(-231, 'Data questionable;Lower window log error', 'This indicates that a difference measurement in the lower window has given a negative result when the units of measurement were logarithmic.'),
        error_item(-231, 'Data questionable;Upper window log error', 'This indicates that a difference measurement in the upper window has given a negative result when the units of measurement were logarithmic.'),
        error_item(-231, 'Data questionable;ZERO ERROR', 'Power meter zeroing failed. The most likely cause is attempting to zero when some power signal is being applied to the power sensor.'),
        error_item(-241, 'Hardware missing', 'The power meter is unable to execute the command because either no power sensor is connected or it expects an Agilent E-Series or N8480 Series power sensor, and one is not connected.'),
        error_item(-310, 'System error;Dty Cyc may impair accuracy with ECP sensor', 'This indicates that the sensor connected is for use with CW signals only.'),
        error_item(-310, 'System error;Sensor EEPROM Read Failed - critical data not found or unreadable', 'This indicates a failure with your Agilent E-Series or N8480 Series power sensor. Refer to your power sensor manual for details on returning it for repair.'), 
        error_item(-310, 'System error;Sensor EEPROM Read Completed OK but optional data block(s) not found or unreadable', 'This indicates a failure with your Agilent E-Series or N8480 Series power sensor. Refer to your power sensor manual for details on returning it for repair.'),
        error_item(-310, 'System error;Sensor EEPROM Read Failed - unknown EEPROM table format', 'This indicates a failure with your Agilent E-Series or N8480 Series power sensor. Refer to your power sensor manual for details on returning it for repair.'),
        error_item(-310, 'System error;Sensor EEPROM < > data not found or unreadable', 'Where < > refers to the sensor data block covered, for example, Linearity, Temp - Comp (temperature compensation). This indicates a failure with your Agilent E-Series or N8480 Series power sensor. Refer to your power sensor manual for details on returning it for repair.'),
        error_item(-310, 'System error;Option 001 Battery charger fault', 'The power meter is connected to an AC power source, the battery is not fully charged and it is not charging.'),
        error_item(-310, 'System error;Sensors connected to both front and rear inputs. You cannot connect two power sensors to the one channel input. In this instance, the power', 'meter detects power sensors connected to both its front and rear channel inputs.'),
        error_item(-320, 'Out of memory', 'The power meter required more memory than was available to run an internal operation.'),
        error_item(-330, 'Self-test Failed;', 'The -330, "Self-test Failed" errors indicate that you have a problem with your power meter. Refer to "Contacting Agilent Technologies" on page 103 for details of what to do with your faulty power meter.'),
        error_item(-330, 'Self-test Failed;Measurement Channel Fault', 'Refer to "Measurement Assembly" on page 98 if you require a description of the Measurement Assembly test.'),
        error_item(-330, 'Self-test Failed;Option 001 Battery requires replacement', 'The Option 001 battery is not charging to a satisfactory level and should be replaced.'),
        error_item(-330, 'Self-test Failed;RAM Battery Fault', 'Refer to "RAM Battery" on page 98 if you require a description of the battery test. '),
        error_item(-330, 'Self-test Failed;Calibrator Fault', 'Refer to "Calibrator" on page 99 if you require a description of the calibrator test. '),
        error_item(-330, 'Self-test Failed;ROM Check Failed', 'Refer to "ROM Checksum" on page 98 if you require a description of the ROM Checksum test. '),
        error_item(-330, 'Self-test Failed;RAM Check Failed', 'Refer to "RAM" on page 98 if you require a description of the RAM test. '),
        error_item(-330, 'Self-test Failed;Display Assy. Fault', 'Refer to "Display" on page 99 if you require a description of the Display test. '),
        error_item(-330, 'Self-test Failed;Confidence Check Fault', 'Refer to "Confidence Check" on page 96 if you require a description of this test. '),
        error_item(-330, 'Self-test Failed;Serial Interface Fault', 'Refer to "Serial Interface" on page 99 if you require a description of this test. '),
        error_item(-350, 'Queue overflow', 'The error queue is full and another error has occurred which could not be recorded.'),
        error_item(-361, 'Parity error in program', 'The serial port receiver has detected a parity error and consequently, data integrity cannot be guaranteed.'),
        error_item(-362, 'Framing error in program', 'The serial port receiver has detected a framing error and consequently, data integrity cannot be guaranteed.'),
        error_item(-363, 'Input buffer overrun', 'The serial port receiver has been overrun and consequently, data has been lost.'),
        error_item(-410, 'Query INTERRUPTED', 'A command was received which sends data to the output buffer, but the output buffer contained data from a previous command (the previous data is not overwritten). The output buffer is cleared when power has been off or after *RST (reset) command has been executed.'),
        error_item(-420, 'Query UNTERMINATED', 'The power meter was addressed to talk (that is, to send data over the interface) but a command has not been received which sends data to the output buffer. For example, you may have executed a CONFigure command (which does not generate data) and then attempted to read data from the remote interface.'),
        error_item(-430, 'Query DEADLOCKED', 'A command was received which generates too much data to fit in the output buffer and the input buffer is also full. Command execution continues but data is lost. -440 Query UNTERMINATED after indefinite response The *IDN? command must be the last query command within a command string.'),
    ]
    
    @classmethod
    def check(cls, num, msg):
        if num==0: return
        for e in cls.error_list:
            if msg == e.msg:
                emsg = '%s (%d)'%(e.msg, e.num)
                msg = 'Power meter returned Error message.\n'
                msg += '*'*len(emsg) + '\n'
                msg += emsg + '\n'
                msg += '*'*len(emsg) + '\n'
                msg += e.txt + '\n'
                raise StandardError(msg)
            continue
        _msg = 'Power meter returned Error message.\n'
        emsg = '%s (%d)\n'%(msg, num)
        _msg += '*'*len(emsg) + '\n'
        _msg += emsg
        _msg += '*'*len(emsg) + '\n'
        raise StandardError(_msg)
        return
    
    



from __future__ import print_function
import logging
import os
import subprocess
import sys
import gphoto2 as gp
import time
import socket

#time1 = time.ctime()
#time2 = time.strptime(time1)
#time3 = time.strftime('%Y%m%d_%H.%M.%S', time2)
timestr = time.strftime('%Y%m%d_%H.%M.%S', time.strptime(time.ctime()))

#savedir_pre = '/home/1.85m/evaluation/optical_pointing/test/fig/'

HOST = '192.168.100.12'
PORT = 50000

#def capture(savedir, imagename):
#f = open('%s%s'%(savedir, imagename), 'w')
#f.write('test')
#f.close()

class m100(object):
    def capture(self,savepath):
        #savedir = './picture/'+timestr+'.JPG'
        savedir = savepath
        logging.basicConfig(format='%(levelname)s: %(name)s: %(massage)s', level=logging.WARNING)
        gp.check_result(gp.use_python_logging())
        camera = gp.check_result(gp.gp_camera_new())
        gp.check_result(gp.gp_camera_init(camera))
        print('Capturing image')
        file_path = gp.check_result(gp.gp_camera_capture(camera, gp.GP_CAPTURE_IMAGE))
        #print('Camera file path: {0}/{1}'.format(file_path.folder, file_path.name))
        #target = os.path.join(savedir, imagename)
        #print('Copying image to', target)
        camera_file = gp.check_result(gp.gp_camera_file_get(camera, file_path.folder, file_path.name, gp.GP_FILE_TYPE_NORMAL))
        #gp.check_result(gp.gp_file_save(camera_file, target))
        #timestr = time.strftime('%Y%m%d_%H.%M.%S', time.strptime(time.ctime()))
        gp.check_result(gp.gp_file_save(camera_file, savedir))
        #gp.check_result(gp.gp_file_save(camera_file, './picture/'+timestr+'.jpg'))
        #subprocess.call(['xdg-open', target])
        gp.check_result(gp.gp_camera_exit(camera))
        return 'Shooting completed'
        #return 0

'''
s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
s.bind((HOST, PORT))
s.listen(1)
print('Server is listening for connections')

While True:
    conn, addr = s.accept()
    print('connected')
    capture()
    bytes = open('./picture/'+timestr+'.JPG', encoding='utf8', errors='ignore').read()
    print(len(bytes), 'bytes')
    bytes += '¥n'
    _file = open('./picture/'+timestr+'.JPG', 'wb')
    _file.write(bytes)
    _file.close()
    conn.send(bytes)
'''



import setuptools
from pathlib import Path


long_description = (Path(__file__).parent / "README.md").read_text()

setuptools.setup(
    name = 'ogameasure',
    version = "0.5.3",
    description = 'Driver for SCPI device',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = 'https://github.com/ogawa-ros/ogameasure',
    author = 'Shota Ueda',
    author_email = 's7u27astr0@gmail.com',
    license = 'MIT',
    keywords = '',
    install_requires=[
        "importlib-metadata; python_version < '3.8'",
        "numpy",
        "pyserial",
    ],
    packages = [
        'ogameasure',
        'ogameasure.communicator',
        'ogameasure.device',
        'ogameasure.device.Agilent',
        'ogameasure.device.Anritsu',
        'ogameasure.device.Pfeiffer',
        'ogameasure.device.Lakeshore',
        'ogameasure.device.SCPI',
        'ogameasure.device.ELVA1',
        'ogameasure.device.Phasematrix',
        'ogameasure.device.HEIDENHAIN',
        'ogameasure.device.KIKUSUI',
        'ogameasure.device.Canon',
        'ogameasure.device.Cosmotechs',
        'ogameasure.device.TandD',
        'ogameasure.device.SENA',
        'ogameasure.interface',
    ],
    classifiers=[
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Hardware',
    ],
)



import time
from ..SCPI import scpi

# main class
# ==========

class E8257(scpi.scpi_family):
    manufacturer = 'Agilent'
    product_name = 'E8257'
    classification = 'Signal Generator'
    
    _scpi_enable = '*CLS *ESE *ESE? *ESR? *IDN? *OPC *OPC? *PSC? *RCL ' +\
                   '*RST *SAV *SRE *SRE? *STB? *TST? *WAI'
    
    def _error_check(self):
        #err_num, err_msg = self.error_query()
        #error_handler.check(err_num, err_msg)
        return
    
    def freq_set(self, freq, unit='GHz'):
        """
        FREQ : Set CW frequency
        -----------------------
        This command sets the signal generator output frequency for the CW
        frequency mode.
        
        Args
        ====
        < freq : float :  >
            A frequency value.
        
        < unit : str : 'GHz','MHz','kHz','Hz' >
            Specify the units of the <freq>.
            'GHz', 'MHz', 'kHz' or 'Hz'. default = 'GHz'
        
        Returnes
        ========
        Nothing.
        
        Examples
        ========
        >>> s.freq_set(1, 'GHz')
        >>> s.freq_set(1.234, 'MHz')
        >>> s.freq_set(98765.4321, 'kHz')
        """
        self.com.send('FREQ %.10f %s'%(freq, unit))
        return
    
    def freq_query(self):
        """
        FREQ? : Query CW frequency
        --------------------------
        This command query the signal generator output frequency for the CW
        frequency mode.
        
        Args
        ====
        Nothing.
        
        Returnes
        ========
        < freq : float :  >
            A frequency value in Hz.
        
        Examples
        ========
        >>> s.freq_get()
        +2.0000000e+10
        """
        self.com.send('FREQ?')
        ret = self.com.readline()
        ret = float(ret)
        return ret
        
    def power_set(self, pow, unit='dBm'):
        """
        POW : Set RF output power
        -------------------------
        This command sets the RF output power.
        
        Args
        ====
        < pow : float :  >
            A output power value.
        
        < unit : str : 'dBm'>
            Specify the units of the <pow>.
            'dBm'. default = 'dBm'
        
        Returnes
        ========
        Nothing.
        
        Examples
        ========
        >>> s.power_set(-130)
        >>> s.power_set(-10.2, 'dBm')
        """
        self.com.send('POW %f %s'%(pow, unit))
        return
    
    def power_query(self):
        """
        POW? : Query RF output power
        ----------------------------
        This command query the RF output power.
        
        Args
        ====
        Nothing.
        
        Returnes
        ========
        < power : float :  >
            A power value in dBm.
        
        Examples
        ========
        >>> s.freq_get()
        10.1
        """
        self.com.send('POW?')
        ret = self.com.readline()
        ret = float(ret)
        return ret
    
    def output_set(self, output):
        """
        OUTP : Enable/disable RF output
        -------------------------------
        This command enables or disables the RF output. Although you can
        configure and engage various modulations, no signal is available at
        the RF OUTPUT connector until this command is executed.
        
        Args
        ====
        < output : str,int : 'ON','OFF',1,0 >
            Enable/disable the RF output.
        
        Returnes
        ========
        Nothing.
        
        Examples
        ========
        >>> s.output_set('ON')
        >>> s.output_set(1)
        >>> s.output_set('OFF')
        >>> s.output_set(0)
        """
        self.com.send('OUTP %s'%(str(output)))
        return
    
    def output_on(self):
        """
        (Helper method) OUTP : Enable the RF output
        -------------------------------------------
        This command enables the RF output.
        
        Args
        ====
        Nothing.
        
        Returnes
        ========
        Nothing.
        
        Examples
        ========
        >>> s.output_on()
        """
        self.com.send('OUTP ON')
        return
    
    def output_off(self):
        """
        (Helper method) OUTP : Disable the RF output
        --------------------------------------------
        This command disables the RF output.
        
        Args
        ====
        Nothing.
        
        Returnes
        ========
        Nothing.
        
        Examples
        ========
        >>> s.output_off()
        """
        self.com.send('OUTP OFF')
        return
    
    def output_query(self):
        """
        OUTP? : Query RF output status
        ------------------------------
        This command queriesthe RF output status.
        
        Args
        ====
        Nothing.
        
        Returnes
        ========
        < output : int : 1,0 >
            Enable/disable the RF output.
            1 = ON, 0 = OFF
        
        
        Examples
        ========
        >>> s.output_query()
        1
        """
        self.com.send('OUTP?')
        ret = self.com.readline()
        ret = int(ret)
        return ret
    


class E8257D(E8257):
    product_name = 'E8257'
    




# ==============
# Helper Classes
# ==============

# Error Class
# ===========
class error_item(object):
    num = 0
    msg = ''
    txt = ''
    
    def __init__(self, num, msg, txt):
        self.num = num
        self.msg = msg
        self.txt = txt
        pass

class error_handler(object):
    error_list = []
    
    @classmethod
    def check(cls, num, msg):
        if num==0: return
        for e in cls.error_list:
            if msg == e.msg:
                emsg = '%s (%d)'%(e.msg, e.num)
                msg = 'Signal Generator returned Error message.\n'
                msg += '*'*len(emsg) + '\n'
                msg += emsg + '\n'
                msg += '*'*len(emsg) + '\n'
                msg += e.txt + '\n'
                raise StandardError(msg)
            continue
        _msg = 'Signal Generator returned Error message.\n'
        emsg = '%s (%d)\n'%(msg, num)
        _msg += '*'*len(emsg) + '\n'
        _msg += emsg
        _msg += '*'*len(emsg) + '\n'
        raise StandardError(_msg)
        return
    
    
